#include "stdafx.h"
#include "Graph2D.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

	//  ,   
const double dXLeft=0.2;		//   
const double dYTop =0.2;
const double dFracX=0.6;		//   
const double dFracY=0.6;

CGraph::CGraph (CRect r, UINT Pts)
{
	rect = r;					//    
								//   
	nPoints = Pts<=1000 ? Pts : 1000;
								//  
	TextFont.CreatePointFont(80,"MS Sans Serif");
	TitleFont.CreatePointFont(120,"Times New Roman Cyr");
	PowerFont.CreatePointFont(80,"Arial");
}

void CGraph::ScaleX (TValues& x)
{					//   
	UINT i;
	for (i=0, dMaxX=x[0], dMinX=x[0]; i<nPoints; i++)
	{				//  (min, max)
		dMinX = x[i]<dMinX ? x[i] : dMinX;
		dMaxX = x[i]>dMaxX ? x[i] : dMaxX;
	}
	dSpanX = dMaxX - dMinX;			// 
					//    (0,1)     10
	Scale (dSpanX, iPowerX, dStepX);
					//  () 
	double step = dStepX*pow(10,iPowerX);
					//    1  2  (human factor)
	nStepsX=(UINT)(dSpanX/step) + (dMinX ? 2 : 1);
					//    
	nStepX =(UINT) (dFracX * rect.Width() / nStepsX +.5);
					//  x (dStartX)    
	double d = dMinX/step;
	dStartX = d ? step * ceil (d-1.) : 0.;	// Human factor
	MinX = (UINT) (dXLeft * rect.Width());	//  x ()
	MaxX = MinX + nStepsX * nStepX;			//  x ()
	X = new UINT [nPoints];					//   ()

					//    ()
	for (i=0; i<nPoints; i++)
		X[i] = MinX+(UINT)((x[i]-dStartX)*nStepX/step);
					//    ()
	dStartX /= pow (10,iPowerX);
}

void CGraph::ScaleY (TValues& y)
{	//  ,    .       
	UINT i;
	for (i=0, dMaxY=y[0], dMinY=y[0]; i<nPoints; i++)
	{
		dMinY = y[i]<dMinY ? y[i] : dMinY;
		dMaxY = y[i]>dMaxY ? y[i] : dMaxY;
	}
	dSpanY = dMaxY-dMinY;
	Scale (dSpanY, iPowerY, dStepY);
	double step = dStepY*pow(10,iPowerY);
	
	nStepsY= (UINT)(dSpanY/step) + (dMinY ? 2 : 1);
	nStepY = (UINT) (dFracY * rect.Height() / nStepsY +.5);
					//     (human factor)
	dStartY = step * (ceil (dMaxY/step));
	
	MinY = (UINT) (dYTop * rect.Height());
	MaxY = MinY + nStepsY * nStepY;
	Y = new UINT [nPoints];
	for (i=0; i<nPoints; i++)
		Y[i] = MinY+(UINT)((dStartY-y[i])*nStepY/step);
	dStartY /= pow (10,iPowerY);
}

void CGraph::Scale (double span, int& pow, double& step)
{										//   ()  
	pow=0;								//     (1,10)
	if (0.<span && span<1.)
		for ( ;  span<1.;  span*=10., pow--)
			;							//   10 
	else if (span>10.)					//   10
		for ( ;  span>10.; span/=10., pow++)
			;
	if (span<1.1)						//     
		step=.1;						//   
	else if (span<2.1)
		step=.2;
	else if (span<2.6)
		step=.25;
	else if (span<5.1)
		step=.5;
	else
		step=1.;
}

CGraph::~CGraph ()
{
	delete [] X;	//    , 
	delete [] Y;	//    
}

void CGraph::DrawGrid (CDC* dc, 
					CString& Title, CString& Xlab, CString& Ylab,
					TValues& xGr, TValues& yGr)
{
									//     
	dc->SelectObject(&TextFont);	//  
	TEXTMETRIC tm;					//  
	dc->GetTextMetrics (&tm);
	
	ScaleX (xGr);			//  
	ScaleY(yGr);			//  
	
	UINT x, y, i,						//  
		LW=tm.tmMaxCharWidth,			//   
		LH=tm.tmHeight,					//   
		h2=LH/2,		h4=LH/4,		//   ( )
		w2=LW/2,		w4=LW/4,		//  
		ww=LW+LW,		hh=LH+LH,		//    
		xy=MinX-ww,						// X-   Y
		yx=MaxY+h2,						// Y-   X
		xh=MaxX+ww,						// X-   X
		yh=MinY-hh;						// Y-   Y
	double d;								//  
	CString s;
	
	dc->SetBkMode(TRANSPARENT);			//    
	dc->SetTextAlign(TA_CENTER);		//    

	for (i=0, x=MinX, d=dStartX;		//   X
			i<=nStepsX;
			x+=nStepX, d+=dStepX, i++)
	{
		dc->MoveTo(x,MinY);
		dc->LineTo (x,MaxY);			//   X
		s.Format("%3.1f",d);
		dc->TextOut (x, yx, s);			//    X
	}
	
	for (i=0, y=MinY, d=dStartY;		//   Y
			i<=nStepsY;
			y+=nStepY, d-=dStepY, i++)
	{
		dc->MoveTo(MinX, y);
		dc->LineTo(MaxX, y);
		s.Format("%5.1f",d);
		dc->TextOut(xy,y-h2,s);
	}
									//    
	s.Format("Range =  (%.3g , %.3g)",dMinY,dMaxY);
	dc->TextOut((MinX+MaxX)/2,yh+h2,s);
									//   
	dc->TextOut (MinX+w2,yh-h2,Ylab);
	dc->TextOut (xh+w2,yx,Xlab);
									//    
	dc->SetTextColor (RGB(0,0,255));
	dc->SelectObject(&PowerFont);
									//     Y
	dc->RoundRect(xy-w2-w2,yh-LH,xy+LW+w2,yh+LH,h2,h2);
	dc->TextOut(xy,yh-h4,"10");
									//     X
	dc->RoundRect(xh-w2-w2,yx-hh-h2,xh+LH+w4,yx-h2,h2,h2);
	dc->TextOut(xh,yx-hh+h4,"10");
									//    Y  X
	s.Format("%d",iPowerY);
	dc->TextOut(xy+w2+w4,yh-LH,s);
	s.Format("%d",iPowerX);
	dc->TextOut(xh+w2+w4,MaxY-hh,s);
			//        
	dc->SelectObject(&TitleFont);
	dc->TextOut((MinX+MaxX)/2,yh-ww,Title);
}
							//   Y (X)
void CGraph::DrawFunc (CDC* dc)
{							//     
	dc->MoveTo(X[0], Y[0]);
							//     
	for (UINT i=0;  i < nPoints; i++)
		dc->LineTo (X[i], Y[i]);
}

extern "C" __declspec(dllexport)
double Aperiodic(double x, TData* p)
{						//   
						//     
	double T1=p->d1, T2=p->d2, K=p->d3;
						//     
	double TT=T1-T2;
	return TT ? K * (exp(-x/T1) - exp(-x/T2))/TT
					: K * x * exp(-x/T1)/(T1*T1);
}

extern "C" __declspec(dllexport)
double Oscillating(double x,  TData* p)
{				//   
	double T=p->d1, Ksi=p->d2, K=p->d3;
				// T   , Ksi   
	double d = sqrt(1.-Ksi*Ksi);
	return K * exp(-Ksi*x/T) * sin (d*x/T) / (T*d);
}

extern "C" __declspec(dllexport)
double Gauss(double x, TData* p)
{		//     
		// M   , Sigma  
	double M=p->d1, Sigma=p->d2;
	x = (x-M)/Sigma;
	return exp(-x*x/2.)/(Sigma*sqrt(6.2831853718));
}

extern "C" __declspec(dllexport)
double Poisson(double x, TData* p)
{		//   
		// M   
	double M=p->d1;
	double mk, m = M;
	ULONG i, k;
	if (x<0 || M==0)
		return 0.;
	k = unsigned(x);			// k -  x
	for (i=2, mk=m; i<=k; i++)	// mk = M^k/k! 
		mk *= m/i;
	return  mk*exp(-m);
}
