// TestDoc.cpp : implementation of the CTestDoc class
//

#include "stdafx.h"
#include "Test.h"

#include "TestDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTestDoc

IMPLEMENT_DYNCREATE(CTestDoc, CDocument)

BEGIN_MESSAGE_MAP(CTestDoc, CDocument)
	//{{AFX_MSG_MAP(CTestDoc)
	ON_COMMAND(ID_EDIT_FUNCTION, OnEditFunction)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestDoc construction/destruction

		//       (min, max)
#define getrand(min, max)  \
	(min)+double(rand())*((max)-(min))/32767

CTestDoc::CTestDoc()
{							//   
	nFunc = 4;
	sFunc[0]="Aperiodic Response";		//  
	sFunc[1]="Oscillating Response";
	sFunc[2]="Gauss Distribution";
	sFunc[3]="Poisson Distribution";
	pFunc[0] = Aperiodic;				//  
	pFunc[1] = Oscillating;
	pFunc[2] = Gauss;
	pFunc[3] = Poisson;
	data.d1 = 0.1;						//  
	data.d2 = 0.1;
	data.d3 = 1.;
}

CTestDoc::~CTestDoc()
{
}

BOOL CTestDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	NPoints = 300;						//  
	xGr.SetSize (NPoints);				//  
	yGr.SetSize (NPoints);
	Time=getrand(5.,15.);				//   
	int i =  rand()%4;					//   
	sCurFunc = sFunc[i];				//  
	pCurFunc = pFunc[i];				//  
	switch (i)
	{						//     
	case 0:					//    
		data.d1 = getrand(1e-2,1.);		// 
		data.d2 = getrand(1e-2,1.);		//  
		data.d3 = getrand(1.,100.);		//  
		break;
	case 1:								//  ,  
		data.d1 = getrand(1e-1,0.5);	// 
		data.d2 = getrand(1e-3,.9);		//  
		data.d3 = getrand(1.,100.);
		break;
	case 2:
		data.d1 = getrand(1e-2,9.);		//  
		data.d2 = getrand(1e-2,.9);		// 
		break;
	case 3:
		data.d1 = getrand(1.,9.);		//  
		break;							// 
	}
		//          Output-Debug
	TRACE("\n%s  d1=%g  d2=%g  d3=%g",sCurFunc,data.d1,data.d2,data.d3);
	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CTestDoc serialization

void CTestDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
	}
	else
	{
	}
}

/////////////////////////////////////////////////////////////////////////////
// CTestDoc diagnostics

#ifdef _DEBUG
void CTestDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CTestDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTestDoc commands

void CTestDoc::DeleteContents() 
{
	xGr.RemoveAll();				//  ,
	yGr.RemoveAll();				//  
	CDocument::DeleteContents();
}

void CTestDoc::Calc()
{
	UINT i;
	double x, dx = Time/(NPoints-1);	//  

	for (i=0, x=0.; i<NPoints; i++, x+=dx)
	{								//  
		xGr[i] = x;
				//      
		yGr[i] = pCurFunc(x,&data);
	}
}

void CTestDoc::OnEditFunction()
{
	CFuncDlg dlg(this, data);	//   
	UINT nOld = NPoints;
	if (dlg.DoModal()!=IDOK) 
		return;
								//    
	for (UINT i=0; i<nFunc && sCurFunc!=sFunc[i]; i++)
		;
	pCurFunc = pFunc[i];		//   
	if (NPoints != nOld)
	{
		DeleteContents();
		xGr.SetSize (NPoints);
		yGr.SetSize (NPoints);
	}
	UpdateAllViews(NULL);		//  
}

/////////////////////////////////////////////////////////////////////////////
// CFuncDlg

							//      
BEGIN_MESSAGE_MAP(CFuncDlg, CDialog)
	ON_BN_CLICKED(IDC_PARM, OnParm)
END_MESSAGE_MAP()

CFuncDlg::CFuncDlg(CTestDoc* p, TData& d)
				: CDialog(IDD_FUNC, NULL), data(d)
{					//     
	pDoc = p;		//      
}

void CFuncDlg::DoDataExchange(CDataExchange* pDX)
{					//     
	CDialog::DoDataExchange(pDX);
	DDX_CBString(pDX, IDC_FUNC, pDoc->sCurFunc);
	DDX_Text(pDX, IDC_TIME, pDoc->Time);
	DDV_MinMaxDouble(pDX, pDoc->Time, 1e-4, 1e2);
	DDX_Text(pDX, IDC_N, pDoc->NPoints);
	DDV_MinMaxUInt(pDX, pDoc->NPoints, 2, 1000);
}

BOOL CFuncDlg::OnInitDialog()
{			//     
			//  .
	CComboBox *p = (CComboBox *)GetDlgItem(IDC_FUNC);
	for (UINT i=0; i<pDoc->nFunc; i++)
		p->AddString(pDoc->sFunc[i]);
	CDialog::OnInitDialog();
	return TRUE;
}

void CFuncDlg::OnParm()
{			//     Parameters
	int sel = ((CComboBox*)GetDlgItem(IDC_FUNC))->GetCurSel();
	switch (sel)
	{		//       Combo Box 
			//     
	case 0: CAperDlg(this).DoModal();	break;
	case 1: COscDlg(this).DoModal();	break;
	case 2: CGaussDlg(this).DoModal();	break;
	case 3: CPoisDlg(this).DoModal();	break;
	}
}

/////////////////////////////////////////////////////////////////////////////
// COscDlg

COscDlg::COscDlg(CFuncDlg* p)
	: CDialog(IDD_OSC, NULL) { pDlg = p; }

void COscDlg::DoDataExchange(CDataExchange* pDX) {
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_D1, pDlg->data.d1);
	DDV_MinMaxDouble(pDX, pDlg->data.d1, 1e-7, 10.);
	DDX_Text(pDX, IDC_D2, pDlg->data.d2);
	DDV_MinMaxDouble(pDX, pDlg->data.d2, 1e-7, .9999999);
	DDX_Text(pDX, IDC_D3, pDlg->data.d3);
	DDV_MinMaxDouble(pDX, pDlg->data.d3, 1e-7, 1e7);
}

/////////////////////////////////////////////////////////////////////////////
// CAperDlg

CAperDlg::CAperDlg(CFuncDlg* p)
	: CDialog(IDD_APER, NULL){	pDlg = p;}

void CAperDlg::DoDataExchange(CDataExchange* pDX){
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_D1, pDlg->data.d1);
	DDV_MinMaxDouble(pDX, pDlg->data.d1, 1e-7, 10.);
	DDX_Text(pDX, IDC_D2, pDlg->data.d2);
	DDV_MinMaxDouble(pDX, pDlg->data.d2, 1e-7, 10.);
	DDX_Text(pDX, IDC_D3, pDlg->data.d3);
	DDV_MinMaxDouble(pDX, pDlg->data.d3, 1e-7, 1e7);
}

/////////////////////////////////////////////////////////////////////////////
// CGaussDlg

CGaussDlg::CGaussDlg(CFuncDlg* p)
	: CDialog(IDD_GAUSS, NULL){pDlg = p;}

void CGaussDlg::DoDataExchange(CDataExchange* pDX){
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_D1, pDlg->data.d1);
	DDV_MinMaxDouble(pDX, pDlg->data.d1,-5.,5.);
	DDX_Text(pDX, IDC_D2, pDlg->data.d2);
	DDV_MinMaxDouble(pDX, pDlg->data.d2, 1e-7, 10.);	
}

/////////////////////////////////////////////////////////////////////////////
// CPoisDlg

CPoisDlg::CPoisDlg(CFuncDlg* p)
: CDialog(IDD_POIS, NULL) { pDlg = p; }

void CPoisDlg::DoDataExchange(CDataExchange* pDX) {
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_D1, pDlg->data.d1);
	DDV_MinMaxDouble(pDX, pDlg->data.d1,1.,10.);
}
